#ifndef _EnergyProfile_h_
#define _EnergyProfile_h_
#include <string>
#include <sstream>
#include <iostream>
#include "generatormt.h"
#include "jednostki.h"

using namespace std;

////////////////////////////////////////////////////////////////////////////////
/// EnergyProfile shoots a double precision number whose probability density is
/// given by the parameters to the constructor
////////////////////////////////////////////////////////////////////////////////

class EnergyProfile
{ private:
     int n;
     double Emin;
     double Emax;
     double *prob;
  public:
     ~EnergyProfile(){if (prob) delete [] prob;}

     /// Beam of Zero Energy
     EnergyProfile():n(0),Emin(0),Emax(0),prob(NULL){}

     /// Monoenergetic beam
     EnergyProfile(double E):n(0),Emin(E),Emax(E),prob(NULL){}

     /// uniform probability density betweeen E1 and E2 
     EnergyProfile(double E1,double E2):n(1),Emin(E1),Emax(E2),prob(NULL){}

     /// energy profile given by a histogram encoded in a string
     /// the string contains space separated values:
     /// Emin Emax b1 b2 ... bn 
     /// where b1 b2 ... bn are the heightsof the bars of the histogram
     /// the widths are assumed equal to (Emax-Emin)/n 
     EnergyProfile(string s):n(0),Emin(0),Emax(0),prob(NULL) {read(s);}

     /// the auxiliary helper function for parsing the string paramater to 
     /// the EnergyProfile constructor  
     void read(string s);

     /// a random number conforming to the probability profile
     /// specified by construcor parameters
     double shoot();

     /// diagnostic function for verifying if the beam 
     /// has been correctly constructed
     double print(); 
};  // Energy Profile class

////////////////////////////////////////////////////////////////////////////////
///              I M P L E M E N T A I O N
////////////////////////////////////////////////////////////////////////////////

/// parse the string parameter to the EnergyProfile constructor  
inline void EnergyProfile::read(string s)
     {
      if(prob) 
        delete[] prob;
      prob=NULL;
      Emin=Emax=n=0; 
      
      stringstream in(s);
      in>>Emin>>Emax;
      Emin*=MeV;
      Emax*=MeV;
      if(!in) 
        return;
	
      n=1;	
      double bufor[500];	
      int i=0;
      while(in>>bufor[i])
         i++;
      if(i>1)
        {n=i;
	 prob=new double[n+1];
	 prob[0]=0;
	 for(int i=1;i<=n;i++) 
	   prob[i]=prob[i-1]+bufor[i-1];
	}   
     }

/// yield a random number with the probability profile
/// specified by the constructor parameters
inline double EnergyProfile::shoot() 		 
     { if(n==0) return Emin;
       if(n==1) return Emin+frandom()*(Emax-Emin);	  
       double x=frandom()*prob[n];
       int i=n;
       while(x<prob[--i]);
       return Emin+(i+frandom())*(Emax-Emin)/n;      
     }

/// diagnostic function for verifying if the beam 
/// has been correctly constructed
inline double EnergyProfile::print() 		 
     { 
       cout<<"Emin="<<Emin<<endl;
       cout<<"Emax="<<Emax<<endl;
       cout<<"n="<<n<<endl;
       if(n>1)
       for(int i=0;i<n;i++)
       cout<<"prob["<<i<<"]="<<prob[i+1]-prob[i]<<endl;
     }
////////////////////////////////////////////////////////////////////////////////
///        END OF  I M P L E M E N T A I O N
////////////////////////////////////////////////////////////////////////////////

#endif
