#ifndef _Metropolisek_h_
#define _Metropolisek_h_
#include "pdg.h"
#include "particle.h"
#include <iostream>
#include "nucleus.h"
#include <TGenPhaseSpace.h>
#include "jednostki.h"
#include "scatter.h"
#include "proctable.h"

using namespace std;
using namespace PDG;

#define echo(x) cout<<#x<<endl
#undef echo
#define echo(x)  


//////////////////////////////////////////////////////////////////////////
/// This file implements the reactions relevant for pion nucleon 
/// intranuclear cascade described in:
/// Phys. Rev. 110, 204 - 219 (1958), N. Metropolis, R. Bivins, and M. Storm 
/// Monte Carlo Calculations on Intranuclear Cascades. II. High-Energy Studies and Pion Processes
//////////////////////////////////////////////////////////////////////////

enum
{ same_nucleon = 0, different_nucleon = 1 };

/// helper function used by the cross section functions
static double *
nucl_params (double Ek)
{
  Ek /= MeV;
  static double ndata[][13] = 
  { //  E1,  E2, sii1, sii2, sij1, sij2, fii,   fij,  Aii, Aij,Bii,  Bij,  fpi  
    {   0,  335, 24.5, 26.4, 33.0, 34.0, 0.07, 0.04,  0.1, 2.2,  0, -1.0, 1.00},
    { 335,  410, 24.5, 26.4, 33.0, 34.0, 0.07, 0.04,  0.1, 2.2,  0, -1.0, 1.00},
    { 410,  510, 26.4, 30.4, 34.0, 35.1, 0.20, 0.07,  0.9, 1.8,  0, -1.1, 1.00},
    { 510,  660, 30.4, 41.2, 35.1, 36.5, 0.31, 0.15,  2.7, 2.3,  0, -0.7, 1.00},
    { 660,  840, 41.2, 47.2, 36.5, 37.9, 0.43, 0.27,  9.0, 8.8,  0, -0.2, 1.00},
    { 840, 1160, 47.2, 48.0, 37.9, 40.2, 0.58, 0.37, 14.3, 15.0, 0,  0.0, 0.97},
    {1160, 1780, 48.0, 44.2, 40.2, 42.7, 0.65, 0.36, 19.2, 29.4, 0,  0.0, 0.80},
    {1780, 3900, 44.2, 41.0, 42.7, 42.0, 0.69, 0.35, 1e99, 1e99, 0,  0.0, 0.44},
    {3900, 1e99, 41.0, 41.0, 42.0, 42.0, 0.69, 0.35, 1e99, 1e99, 0,  0.0, 0.44}
  };
  for (int i = 0; i < 9; i++)
    if (Ek < ndata[i][1])
      return ndata[i];
  return ndata[0];
}


// helper function used by the cross section functions
static double *
pion_params (double Ek)
{
  Ek /= MeV;
  static double pdata[][21] = 
  {// E1,   E2,sii1,sii2,sij1,sij2,sabs1,sabs2, fxii, fxij,  fx0,fceii,fceij, fce0, Aii, Aij,  A0,  Bii, Bij,   B0, fpi0     
    {  0,   49,  16,  50,  15,  21,   20,   32, 0.00, 0.45, 0.42,    0, 0.80, 0.80, 2.5, 2.5, 3.0, -3.5, 3.5, -2.0, 1.00},
    { 49,   85,  16,  50,  15,  21,   20,   32, 0.00, 0.45, 0.42,    0, 1.00, 1.00, 3.2, 1.1, 3.4, -1.8, 0.8, -1.8, 1.00},
    { 85,  128,  50, 114,  21,  43,   32,   45, 0.00, 0.57, 0.36,    0, 1.00, 1.00, 2.2, 1.9, 2.1, -2.1, 0.7, -2.0, 1.00},
    {128,  184, 114, 200,  43,  66,   45,   36, 0.00, 0.62, 0.36,    0, 1.00, 1.00, 1.9, 2.2, 1.9, -1.5, 0.8, -1.4, 1.00},
    {184,  250, 200, 110,  66,  44,   36,   18, 0.03, 0.64, 0.37,    0, 0.95, 0.90, 2.2, 2.2, 2.1, -0.3, 1.0,  0.0, 1.00},
    {250,  350, 110,  51,  44,  23,   18,    0, 0.06, 0.62, 0.40,    0, 0.89, 0.84, 2.6, 2.0, 2.5,  2.0, 1.4,  1.7, 1.00},
    {350,  540,  51,  20,  23,  22,    0,    0, 0.16, 0.56, 0.50,    0, 0.72, 0.67, 3.0, 2.7, 3.0,  4.0, 2.6,  4.0, 0.98},
    {540, 1300,  20,  30,  22,  30,    0,    0, 0.30, 0.58, 0.59,    0, 0.51, 0.50, 3.0, 3.0, 3.0,  4.0, 3.6,  4.0, 0.91},
   {1300, 1e10,  30,  30,  30,  30,    0,    0, 0.88, 0.94, 0.94,    0, 0.06, 0.05, 3.0, 3.0, 3.0,  4.0, 4.0,  4.0, 0.24}
  };

  for (int i = 0; i < 8; i++)
    if (pdata[i][0] <= Ek && Ek < pdata[i][1])
      return pdata[i];
  return pdata[0];
}				// pion

/// the square of a number (to replace things like t[2][3]*t[2][3] with sqr(t[2][3]))
template < class T >
static inline T sqr (T x)
{
  return x * x;
}

//----------------------------------------------------------------------
/// nucleondata class - interface to  Metropolis cross section
//----------------------------------------------------------------------
struct nucleondata
{
  double E1, E2;
  double s1[2], s2[2], f[2], A[2], B[2], fp;	// 0 = ii , 1 =ij

    nucleondata (double x[])
  {
    E1 = x[0];
    E2 = x[1];
    s1[0] = x[2];
    s2[0] = x[3];
    s1[1] = x[4];
    s2[1] = x[5];
    f[0] = x[6];
    f[1] = x[7];
    A[0] = x[8];
    A[1] = x[9];
    B[0] = x[10];
    B[1] = x[11];
    fp = x[12];
  }

  /// Cross section for nucleon - nucleon scattering   
  /// diff = 0  (nn) or (pp) scattering
  /// diff = 1  (np) or (pn) scattering
  /// Ek = kinetic energy (must be between E1 and E2)
  double sigma (bool diff, double Ek)
  {
    if (Ek < 335 * MeV)
      {
	const double M = (mass_proton + mass_neutron) / 2;
	double E = Ek + M, v = sqrt (1 - sqr (M / E));
	return (diff ? (34.10 / v - 82.2) / v +	82.2 
                     : (10.63 / v - 29.92) / v + 42.9) * millibarn;
      }
    double a = Ek - E1, b = E2 - Ek;
    return (s1[diff] * b + s2[diff] * a) / (a + b) * millibarn;
  }

};// end of class


double nucleon_sigma (bool diff, double Ek)
  { 
    if (Ek < 335 * MeV)
      {
	const double M = (mass_proton + mass_neutron) / 2;
	double E = Ek + M, v = sqrt (1 - sqr (M / E));
	return (diff ? (34.10 / v - 82.2) / v +
		82.2 : (10.63 / v - 29.92) / v + 42.9) * millibarn;
      }
    else
      {double *x=nucl_params(Ek);
       double *s= (diff ? x+4 : x+2);
       //double s[0][0] = x[2];
       //double s[0][1] = x[3];
       //double s[1][0] = x[4];
       //double s[1][1] = x[5];
       double a = Ek - x[0], b = x[1] - Ek;
       
       return (s[0] * b + s[1] * a) / (a + b) * millibarn;
       }
  }




//----------------------------------------------------------------------
/// remember table data for pion reactions
//----------------------------------------------------------------------
struct piondata
{
  double E1, E2;
  double s[3][2], f[3], fc[3], A[3], B[3];	// 0 = ii, 1 = ij, 2 = abs lub 0
  double fp;

    piondata (double a[])
  {
    E1 = a[0];
    E2 = a[1];

    s[0][0] = a[2];
    s[0][1] = a[3];
    s[1][0] = a[4];
    s[1][1] = a[5];
    s[2][0] = a[6];
    s[2][1] = a[7];

    f[0] = a[8];
    f[1] = a[9];
    f[2] = a[10];

    fc[0] = a[11];
    fc[1] = a[12];
    fc[2] = a[13];

    A[0] = a[14];
    A[1] = a[15];
    A[2] = a[16];

    B[0] = a[17];
    B[1] = a[18];
    B[2] = a[19];

    fp = a[20];
  }

  /// Cross section for pion - pion scattering   
  /// k = 0  (pii) scattering
  /// k = 1  (pij) scattering
  /// k = 2  (pi0) scattering
  /// Ek = kinetic energy (must be between E1 and E2)
  double sigma (int k, double Ek)
  {
    if (Ek <= 49 * MeV)
      {	      
        double x = Ek / mass_pi;
        switch (k)
	  {
	  case 0:  return (3.7 + 286 * x * x * x) * millibarn;
	  case 1:  return (6.5 + 23.9 * x) * millibarn;
	  case 2: 
	    {
	      double p = sqrt (sqr (x + 1) - 1);
	      return (16.4 * (0.14 / p + p)) * millibarn;
	    }
	  }
      }
    else
      {
	double a = Ek - E1, b = E2 - Ek;
	return (s[k][0] * b + s[k][1] * a) / (a + b) * millibarn;
      }
  }


  double total_sigma (int k, double Ek)
  {				//if(Ek>=1300*MeV) return 30*millibarn;
    switch (k)
      {
      case 0:  return sigma (0, Ek);	// pii
      case 1:  return sigma (1, Ek) + sigma (2, Ek);	// pij
      case 2:  return (sigma (0, Ek) + sigma (1, Ek) + sigma (2, Ek)) / 2;	// pi0 
      default: return 0;
      }
  }

};				// end of class


//---------------------------------------------------------------------- 
/// Interaction
//---------------------------------------------------------------------- 
class Interaction
{  public: 
    enum {nucleon_=10,pion_=20} k1;
    enum {elastic_=0, ce_=1, spp_=2, dpp_=3, abs_=4} k2;

    const char* process_name()
    {switch(k1)
      {case nucleon_:
       switch(k2)
       {
        case elastic_: return "nucleon elastic";
        case ce_:      return "nucleon ce";
        case spp_:     return "nucleon spp";
        case dpp_:     return "nucleon dpp";
       }
       case pion_:
       switch(k2)
        {
         case elastic_: return "pion elastic";
         case ce_:      return "pion ce";
         case spp_:     return "pion spp";
         case dpp_:     return "pion dpp";
	 case abs_:     return "pion abs";
        }
      }
      return NULL;
    };

    int process_id()
    { return k1+k2;
    }

	
private:
/*
  void set_products (particle p[4], int &n, int k1, int k2, int k3, int k4)
  {
    const int a[] = { 0, pdg_proton, pdg_neutron, pdg_piP, pdg_pi, -pdg_piP };
    const int b[] =
      { 0, mass_proton, mass_neutron, mass_piP, mass_pi, mass_piP };
    n = 0;
    if (k1)
      p[n++].set_pdg_and_mass (a[k1], b[k1]);
    if (k2)
      p[n++].set_pdg_and_mass (a[k2], b[k2]);
    if (k3)
      p[n++].set_pdg_and_mass (a[k3], b[k3]);
    if (k4)
      p[n++].set_pdg_and_mass (a[k4], b[k4]);
  }
*/
public:
  // this function is not used (but works well)
  double cross_section (int pdg1, int pdg2, double Ek, double *chanels = NULL)
  {
    if (nucleon (pdg1))		//p1 is nucleon
      {
	if (pdg1 == pdg2)
	  return nucleondata (nucl_params (Ek)).sigma (same_nucleon, Ek);
	else
	  return nucleondata (nucl_params (Ek)).sigma (different_nucleon, Ek);
      }
    else			//  p1 is pion
      {
	if (Ek >= 1300 * MeV)
	  return 30 * millibarn;

	piondata pd (pion_params (Ek));

	if (pdg1 == pdg_pi)
	  return (pd.sigma (0, Ek) + pd.sigma (1, Ek) + pd.sigma (2, Ek)) / 2;

	if ((pdg1 == pdg_piP) == (pdg2 == pdg_proton))
	  return pd.sigma (0, Ek);
	else
	  return pd.sigma (1, Ek) + pd.sigma (2, Ek);
      }
  };


/// scattering of pion or nucleon on nucleus 
  void
    total_cross_sections(int pdg, double Ek,  double &s0, double &s1)
  {
    switch (pdg)
      {
      case pdg_neutron:
	{
	  nucleondata nd (nucl_params (Ek));
	  s0 = nd.sigma (same_nucleon, Ek);
	  s1 = nd.sigma (different_nucleon, Ek);
	  break;
	}
      case pdg_proton:
	{
	  nucleondata nd (nucl_params (Ek));
	  s0 = nd.sigma (different_nucleon, Ek);
	  s1 = nd.sigma (same_nucleon, Ek);
	  break;
	}
      default:			//jest to pion
	piondata pd (pion_params (Ek));
	double sii = pd.sigma (0, Ek);
	double sij = pd.sigma (1, Ek);
	double sabs = pd.sigma (2, Ek);
	switch (pdg)
	  {
	  case pdg_pi:
	    s0 =  (sii + sij + sabs) / 2;
	    s1 =  (sii + sij + sabs) / 2;
	    break;
	  case pdg_piP:
	    s0 =  sij + sabs;
	    s1 =  sii;
	    break;
	  case -pdg_piP:
	    s0 =  sii;
	    s1 =  sij + sabs;
	    break;
	  default:
	    s0=s1=0;
	  };
      };
  }


/// pion or nucleon scattering on nucleon
/// fracproton - probability that the scattering will be on proton 
  int particle_scattering (particle & p1, particle & p2, double fracproton,
			   int &n, nucleus & t, particle p[])
  {
    echo (particle_scattering);
    int res;
    switch (p1.pdg)
      {
      case pdg_proton:
      case pdg_neutron:
	k1=nucleon_;
	res = nucleon_scattering (p1, p2, fracproton, n, t, p);
//	cout<<"nucleon"<<endl;
	break;
      case pdg_pi:
      case pdg_piP:
      case -pdg_piP:
	k1=pion_;
	res = pion_scattering (p1, p2, fracproton, n, t, p);
//	cout<<"pion"<<endl;
	break;
      default:			// cout<<" only pion and nucleon Interactions are implemented "<<endl;
	return 0;
      }
////    vect a1 (0, 0, 0, 0);
////    vect a0 = p1 + p2;
////    for (int i = 0; i < n; i++)
////      a1 = a1 + p[i];
////    cout << "Interaction: 4p preservation  " << a0 << "=>" << a1 << endl;
    return res;
  }

/// nucleon nucleon scattering
/// fracproton - probability that the scattering will be on proton 
  int nucleon_scattering (particle & p1, particle & p2, double fracproton,
			  int &n, nucleus & t, particle p[])
  {
    //p2=t.shoot(); // rand_from_ball(kf);
    p2 = t.get_nucleon (p1.r);

// cout<<"get_p2="<<p2<<endl;

    double x = frandom11 ();
    if (x < fracproton)
      p2.set_proton ();
    else
      p2.set_neutron ();

    particle p2m = p2;
    p2m.x = -p2.x;
    p2m.y = -p2.y;
    p2m.z = -p2.z;
    double Ek1 = relative_energy (p1, p2) - p1.mass ();
    double Ek2 = relative_energy (p1, p2m) - p1.mass ();
    double Ek = Ek1;

    nucleondata nd1 (nucl_params (Ek1));
    nucleondata nd2 (nucl_params (Ek2));

    nucleondata nd = nd1;
    bool dif = (p1.pdg != p2.pdg);	// 0 for identical particles 1 for different particles
    double s1 = nd1.sigma (dif, Ek1);
    double s2 = nd2.sigma (dif, Ek2);
    double s = s1;

    if (frandom11 () * (s1 + s2) < s2)
      {
	p2.x = -p2.x;
	p2.y = -p2.y;
	p2.z = -p2.z;
	nd = nd2;
	Ek = Ek2;
	s = s2;
      }
    p[0] = p1;
    p[1] = p2;
    p[2].set_pi ();
    p[3] = p[2];

    x = frandom ();
    if (x >= nd.f[dif])
      {				// elastic scattering
	n = 2;
	k2=elastic_;
        int res=scatterAB (p1, p2, p[0], p[1], nd.A[dif], nd.B[dif]);
	if(res==0) cerr<<"AB=0"<<endl;
	return res;
      }
    else if (x < nd.f[dif] * nd.fp)
      {				//spp
	n = 3;
	k2=spp_;
	return nucleon_spp (p1, p2, p);	// p[2] is pion
      }
    else
      {				//dpp
	n = 4;
	k2=dpp_;
	return nucleon_dpp (p1, p2, p);	// p[2],p[3] are pions
      }
  }

/// pion nucleon scattering 
/// fracproton - probability that the scattering will be on proton 
  int pion_scattering (particle & p1, particle & p2, double fracproton,
		       int &n, nucleus & t, particle p[])
  {
    double x = frandom11 ();
    p2 = t.get_nucleon (p1.r);
// cout << "get_p2=" << p2 << endl;
    if (x < fracproton)
      p2.set_proton ();
    else
      p2.set_neutron ();
    particle p2m = p2;
    p2m.x = -p2.x;
    p2m.y = -p2.y;
    p2m.z = -p2.z;
    double Ek1 = relative_energy (p1, p2) - p1.mass ();
    double Ek2 = relative_energy (p1, p2m) - p1.mass ();
    double Ek = Ek1;
    piondata pd1 (pion_params (Ek1));
    piondata pd2 (pion_params (Ek2));
    piondata pd = pd1;
    int i;
    if (p1.pdg = pdg_pi)
      i = 2;
    else
      if ((p1.pdg == pdg_piP && p2.pdg == pdg_proton)
	  || (p1.pdg == -pdg_piP && p2.pdg == pdg_neutron))
      i = 1;
    else
      i = 0;
    double s1 = pd1.total_sigma (i, Ek1);
    double s2 = pd2.total_sigma (i, Ek2);
    double s = s1;
    if (frandom11 () * (s1 + s2) < s2)
      {
	p2.x = -p2.x;
	p2.y = -p2.y;
	p2.z = -p2.z;
	pd = pd2;
	Ek = Ek2;
	s = s2;
      }
    double sabs, sinel;
    switch (i)
      {
      case 0:
	sabs = 0;
	sinel = pd.sigma (0, Ek);
	break;

      case 1:
	sabs = pd.sigma (2, Ek);
	sinel = pd.sigma (1, Ek);
	break;

      case 2:
	sabs = pd.sigma (2, Ek) / 2;
	sinel = (pd.sigma (0, Ek) + pd.sigma (1, Ek)) / 2;
	break;
      }

    p[0] = p1;
    p[1] = p2;
    p[2] = p1;
    p[3] = p1;
    n = 2;
    x = frandom11 () * (sabs + sinel);
    if (x > sinel)		// absorption
      {				//cout<<"pion_abs"<<endl;
	k2=abs_;
	return pion_abs (p1, p2, t, p);
      }
    else if (x > sinel * pd.f[i])	// elastic 
      {				//cout<<"pion_elast"<<endl;
	k2=elastic_;
	return scatterAB (p1, p2, p[0], p[1], pd.A[i], pd.B[i]);
      }
    else if (x > sinel * pd.f[i] * (1 - pd.fc[i]))	// charge exchange 
      {
	k2=ce_;
	{			//cout<<"pion_CE"<<endl;
	  switch (p1.pdg)
	    {
	    case pdg_pi:
	      if (p2.pdg == pdg_proton)
		{
		  p[0].set_piP ();
		  p[1].set_neutron ();
		}
	      else
		{
		  p[0].set_piM ();
		  p[1].set_proton ();
		}
	      break;
	    case pdg_piP:
	      if (p2.pdg == pdg_neutron)
		{
		  p[0].set_pi ();
		  p[1].set_proton ();
		}
	      break;
	    case -pdg_piP:
	      if (p2.pdg == pdg_proton)
		{
		  p[0].set_pi ();
		  p[1].set_neutron ();
		}
	      break;
	    }
	}
	if (Ek > 52 * MeV)
	  return scatterAB (p1, p2, p[0], p[1], pd.A[2], pd.B[2]);
	else
	  return scatterAB (p1, p2, p[0], p[1], 1.5, -2.5);
      }
    else if (x < sinel * pd.f[i] * (1 - pd.fc[i]) * pd.fp)
      {				//spp
	n = 3;
	k2=spp_;
	return pion_spp (p1, p2, p);	// p[0], p[2] are pions
      }
    else			//dpp
      {
	n = 4;
	k2=dpp_;
	return pion_dpp (p1, p2, p);	// p[0], p[2], p[3] are pions
      }
  }

/// nucleon single pion production
  int nucleon_spp (particle p1, particle p2, particle p[])	// p[2] is pion
  {
    echo (nucleon_spp);
    p[0] = p1;
    p[1] = p2;
    double x = frandom ();
    if (p1.pdg == p2.pdg)
      {
	if (x < 0.11)		// pi0 production 
	  p[2].set_pi ();
	else			//  charge exchange pion production 
	if (p1.pdg == pdg_proton)
	  {
	    p[1].set_neutron ();
	    p[2].set_piP ();
	  }
	else
	  {
	    p[1].set_proton ();
	    p[2].set_piM ();
	  }
      }
    else			// different nucleons 
      {
	if (x < 0.43)		// pi0 production
	  p[2].set_pi ();
	else			// charge exchange pion production 
	if (x < 1 - 0.57 / 2)
	  {
	    p[0].set_neutron ();
	    p[1].set_neutron ();
	    p[2].set_piP ();
	  }
	else
	  {
	    p[0].set_proton ();
	    p[1].set_proton ();
	    p[2].set_piM ();
	  }
      }
    return scatter (3, p1, p2, p);
  }

/// pion single pion production
  int pion_spp (particle p1, particle p2, particle p[])
  {
    echo (pion_spp);
    p[0] = p1;
    p[1] = p2;

    double frac;
    if ((p1.pdg == pdg_piP and p2.pdg == pdg_proton)
	 or
         (p1.pdg == -pdg_piP and p2.pdg == pdg_neutron)
	)
      frac = 0.11;
    else
      frac = 0.44;

    double x = frandom11 ();
    if (x < frac)
      {// pi0 production
	p[2].set_pi ();
      }				
    else
      {	// p- and pi+ production			
	if (p2.pdg == pdg_proton)
	  {
	    p[1].set_neutron ();
	    p[2].set_piP ();
	  }
	else
	  {
	    p[1].set_proton ();
	    p[2].set_pi ();
	  }
      }
    return scatter (3, p1, p2, p);
  }


/// nucleon double pion production
  int nucleon_dpp (particle p1, particle p2, particle p[])	// p[2],p[3] are pions
  {
    echo (nucleon_dpp);
    p[0] = p1;
    p[1] = p2;
    p[2].pdg = 1;
    p[3].pdg = 1;
    double x = frandom11 ();
    {
      if (x < 0.80)
	{
	  p[1].pdg = p[0].pdg = p1.pdg;
	  if (x < 0.6)
	    {
	      p[2].set_pi ();
	      p[3].set_pi ();
	    }
	  else
	    {
	      p[2].set_piP ();
	      p[3].set_piM ();
	    }
	}
      else
	{
	  if (p1.pdg == pdg_proton && p2.pdg == pdg_proton)
	    {
	      p[0].set_proton ();
	      p[1].set_neutron ();
	      p[2].set_pi ();
	      p[3].set_piP ();
	    }
	  else if (p1.pdg == pdg_neutron && p2.pdg == pdg_neutron)
	    {
	      p[0].set_neutron ();
	      p[1].set_proton ();
	      p[2].set_pi ();
	      p[3].set_piM ();
	    }
	  else			// by� proton i neutron 
	  if (x < 0.9)		// 50 % przypadk�w 
	    {
	      p[0].set_proton ();
	      p[1].set_proton ();
	      p[2].set_pi ();
	      p[3].set_piM ();
	    }
	  else
	    {
	      p[0].set_neutron ();
	      p[1].set_neutron ();
	      p[2].set_pi ();
	      p[3].set_piP ();
	    }
	}
    }
    return scatter (4, p1, p2, p);
  }

/// pion double pion production
  int pion_dpp (particle p1, particle p2, particle p[])	// p[2], p[3] are pions
  {
    echo (pion_dpp);
    p[0] = p1;
    double x = frandom11 ();
    double frac = 0.25;
    if (x < frac)
      {
	if (x < frac * 0.75)
	  {
	    p[2].set_pi ();
	    p[3].set_pi ();
	  }
	else
	  {
	    p[2].set_piP ();
	    p[3].set_piM ();
	  }
      }
    else
      {
	if (p[1].pdg == pdg_proton)
	  {
	    p[1].set_neutron ();
	    p[2].set_piP ();
	    p[3].set_pi ();
	  }
	else
	  {
	    p[1].set_proton ();
	    p[2].set_pi ();
	    p[3].set_piM ();
	  }
      }
    return scatter (4, p1, p2, p);
  }

/// pion absorption on a pair of nucleons
  int pion_abs (particle p1, particle p2, nucleus & t, particle p[])
  {
    echo (pion_abs);
    particle p2a = t.get_nucleon (p1.r);	// get nucleon from this place in Nucleus
    p[0] = p2;
    p[1] = p2a;
    if (p1.pdg == pdg_piP && p2.pdg == pdg_neutron)
      p[0].set_proton ();
    else if (p1.pdg == -pdg_piP && p2.pdg == pdg_proton)
      p[0].set_neutron ();
    return decay (p1 + p2 + p2a, p[0], p[1]);	// cheeking (number of nucleons not preserved)
  }


/// test functions present in this module
  int metropolis_test ()
  {
    echo (metropolis_test);
    init_genrand (time (NULL));
    particle proton (pdg_proton, mass_proton);
    particle neutron (pdg_pi, mass_pi);
    particle neutron2 (neutron);
    particle proton2 (proton);
    proton.set_momentum (vec (1000, 0, 0));
    neutron.set_momentum (vec (-1000, 0, 0));
    int t[1000];
    for (int i = 0; i < 1000; i++)
      {
	t[i] = 0;
      }
    for (int i = 0; i < 1000000; i++)
      {
	scatterAB (proton, neutron, proton2, neutron2, 100, 0);
	t[int (proton2.y + 1000) / 2]++;
      }
    for (int i = 0; i < 1000; i++)
      {
	cout << i << '\t' << t[i] << endl;
      }
  }
};
#endif
