#ifndef _anynucleus_h_
#define _anynucleus_h_

#include "nucleus.h"
#include "particle.h"
#include "pdg.h"
#include "params.h"
#include "generatormt.h"

#include <algorithm>

using namespace std;

template < class T > static inline T
pow2 (T x)
{
  return x * x;
}

///////////////////////////////////////////////////////////////////////////////
/// this class implements the target nucleus
/// it should take into account of the its radius, charge ,
/// proton and neutron density as well as local fermi momentum and Pauli blocking
/// the number of nucleons can change as a result of scattering and the density 
/// and Fermi momentum should be consistently updated - this is still NOT implemented

/// nucleus concrete class 
/// very accurate density profiles implemented
/// density dependent local Fermi momentum
/// shrinks as nucleons escape 
///////////////////////////////////////////////////////////////////////////////

class anynucleus:public nucleus
{
//  int p;          ///< initial number of protons (inherited)
//  int n;          ///< initial number of neutrons (inherited)
  int pr;			///< real number of protons 
  int nr;			///< real number of neutrons
  double _r;			///< radius
  double Eb;			///< nucleon bounding energy
  double local_kf;		///< local Fermi momentum (if used)
  double kf;			///< global Fermi momentum (if used)
  bool bodek;			///< "bodek tail" i moementum distribution (yes or no)
  double _V;			///< Volume 
  vect missingp4;		///< 4 momentum taken away by escaping nucleons
  
  private:
  double calculate_radius();    ///< needed to initialize r;

  public:

  ///////////////////////////////////////////////////////////////////////////////
  /// Construct nucleus from params given in the input file 
  ///////////////////////////////////////////////////////////////////////////////
    anynucleus (params & par);

  ///////////////////////////////////////////////////////////////////////////////
  /// Potential
  ///////////////////////////////////////////////////////////////////////////////
  double V ()
  {
    return _V;
  }

  ///////////////////////////////////////////////////////////////////////////////
  /// Radius
  ///////////////////////////////////////////////////////////////////////////////
  double radius ()
  {
    return _r;
  }

  ///////////////////////////////////////////////////////////////////////////////
  /// proton density at dist r from the center
  ///////////////////////////////////////////////////////////////////////////////
  double proton_density (double r);

  ///////////////////////////////////////////////////////////////////////////////
  /// neutron density at dist r from the center
  ///////////////////////////////////////////////////////////////////////////////
  double neutron_density (double r);

  ///////////////////////////////////////////////////////////////////////////////
  /// percentage of protons
  ///////////////////////////////////////////////////////////////////////////////
  double frac_proton ()
  {
    return double (pr) / (pr + nr);
  }

  ///////////////////////////////////////////////////////////////////////////////
  /// percentage of neutrons
  ///////////////////////////////////////////////////////////////////////////////
  double frac_neutron ()
  {
    return double (nr) / (pr + nr);
  }

  double prfSOG (double sog[], double r);

  ///////////////////////////////////////////////////////////////////////////////
  /// calculates nuclear density of proton. neutrons at radius r from the center
  ///////////////////////////////////////////////////////////////////////////////
  double density (int pdg = 0, double r = 0);

  ///////////////////////////////////////////////////////////////////////////////
  /// for given p, n find the nearest known  p, n   
  ///////////////////////////////////////////////////////////////////////////////
  int best_pn (int p, int n);

  ///////////////////////////////////////////////////////////////////////////////
  /// random distance from the center waighted with nuclear density
  ///////////////////////////////////////////////////////////////////////////////
  double get_random_r ();

  ///////////////////////////////////////////////////////////////////////////////
  /// local Fermi momentum for particle (pdg and position dependent)
  ///////////////////////////////////////////////////////////////////////////////
  double localkf (particle & pa)
  {
    return localkf (pa.pdg, pa.r.length ());
  }

  ///////////////////////////////////////////////////////////////////////////////
  /// local Fermi momentum from pdg code and  dist r from nucleus center 
  ///////////////////////////////////////////////////////////////////////////////
  double localkf (int pdg, double r)
  {
    switch (pr * 100 + nr)
      {
      case 100:
	return 0;		//wod�r
      default:
	return kf_from_density (density (pdg, r));
      }
    return kf;
  }

  ///////////////////////////////////////////////////////////////////////////////
  /// local Fermi momentum from density 
  ///////////////////////////////////////////////////////////////////////////////
  double kf_from_density (double dens)
  {				// if(dens<=0) return 0;
    return pow (3 * Pi * Pi * dens * (fermi * fermi * fermi) / 2,
		1. / 3.) * 197.4 * MeV;
  }

  ///////////////////////////////////////////////////////////////////////////////
  /// get nucleon with (random) momentum not exceeding localkf(r)
  ///////////////////////////////////////////////////////////////////////////////
  particle shoot (vec r)
  {				//zmieni�
    return get_nucleon (r);	// dummy r dependence
  }
  ///////////////////////////////////////////////////////////////////////////////
  ///  get nucleon with random position and momentum not exceeding localkf(r)
  ///////////////////////////////////////////////////////////////////////////////
  particle shoot ()
  {
    double r = get_random_r ();
    return get_nucleon (r * rand_dir ());
  }

  ///////////////////////////////////////////////////////////////////////////////
  ///  get nucleon with (random) momentum not exceeding localkf(r)
  ///////////////////////////////////////////////////////////////////////////////
  particle get_nucleon (vec r);

  ///////////////////////////////////////////////////////////////////////////////
  /// remove nucleon P from the nucleus 
  ///////////////////////////////////////////////////////////////////////////////
  void remove_nucleon (particle P);

  ///////////////////////////////////////////////////////////////////////////////
  /// insert nucleon P into the nucleus 
  ///////////////////////////////////////////////////////////////////////////////
  void insert_nucleon (particle P);

  ///////////////////////////////////////////////////////////////////////////////
  /// check if particle is Pauli blocked - depends on localkf at particle position
  ///////////////////////////////////////////////////////////////////////////////
  bool pauli_blocking (particle & pa)
  {
    return pauli_blocking (pa.pdg, pa.momentum (), pa.r.length ());
  }

  ///////////////////////////////////////////////////////////////////////////////
  /// check if any of p[0]...p[n-1] is Pauli blocked 
  ///////////////////////////////////////////////////////////////////////////////
  bool pauli_blocking (particle p[], int n);

  ///////////////////////////////////////////////////////////////////////////////
  /// check if particle def by pdg and momentum is blocked at dist r from center
  ///////////////////////////////////////////////////////////////////////////////
  bool pauli_blocking (int pdg, double p, double r);	// TRUE = PARTICLE IS BLOCKED

  ///////////////////////////////////////////////////////////////////////////////
  /// nucleon effective mass in the cuclear matter ar dist r from center
  ///////////////////////////////////////////////////////////////////////////////
  double mstar (double r)
  {				// na razie nie jet to znut mądre
    return mass_proton;
  }


};
#endif
