#ifndef _flatnucleus_h_
#define _flatnucleus_h_

#include "nucleus.h"
#include "particle.h"
#include "pdg.h"
#include "params.h"
#include "generatormt.h"
#include <algorithm>

using namespace std;

/// this class implements the target nucleus
/// it should take into account of the its radius, charge ,
/// proton and neutron density as well as local fermi momentum and Pauli blocking
/// the number of nucleons can change as a result of scattering and the density 
/// and Fermi momentum should be consistently updated - this is still NOT implemented

struct flatnucleus :public nucleus
{
//  int p;  //inherited from nucleus
//  int n; // inherited from nucleus
  double r;			///< radius
  double Eb;			///< binging energy
  double local_kf;		///< bocal Fermi momentum
  double kf;			///< global Fermi moementum
//  double bodek;
  double _V;
  double Volume;
  vect missingp4;


    flatnucleus (params & par):
    nucleus(par), 
    Eb (par.nucleus_E_b * MeV),
  //  local_kf (par.nucleus_local_kf * MeV),
    kf (par.nucleus_kf * MeV)
//bodek (par.nucleus_bodek)
  {
    using namespace PDG;
    if (p + n == 1)
      Eb = kf = r = local_kf = 0;
    double A = n + p;
    double Ap = pow (A, 1.0 / 3);
      _V = sqrt (mass_proton * mass_proton + kf * kf) - mass_proton + 7 * MeV;
      r = Ap *1.2* fermi;		//????
      Volume = 4.0 / 3 * M_PI * r * r * r;
  }


private:


public:

//  int  A(){return n+p;}                   // A

//  int  Z(){return p;}                     // Z

//  int  N(){return n;}                     // N

  ///////////////////////////////////////////////////////////////////////////////
  /// Radius
  ///////////////////////////////////////////////////////////////////////////////
  double radius ()
  {
    return r;
  }
 
  double V(){return _V;}
  ///////////////////////////////////////////////////////////////////////////////
  // proton density at dist r from the center
  ///////////////////////////////////////////////////////////////////////////////
  double proton_density (double r)
  {
    return p / Volume;
  }

  ///////////////////////////////////////////////////////////////////////////////
  // neutron density at dist r from the center
  ///////////////////////////////////////////////////////////////////////////////
  double neutron_density (double r)
  {
    return n / Volume;
  }

  ///////////////////////////////////////////////////////////////////////////////
  /// calculates nuclear density of proton. neutrons at radius r from the center
  ///////////////////////////////////////////////////////////////////////////////
  double density (int pdg = 0, double r = 0)
  {
    switch (pdg)
      {
      case pdg_proton:
	return p / Volume;
      case pdg_neutron:
	return n / Volume;
      case 0:
	return (p + n) / Volume;
      default:
	return 0;
      }

  }

  ///////////////////////////////////////////////////////////////////////////////
  // random distance from the center waighted with nuclear density
  ///////////////////////////////////////////////////////////////////////////////
  double get_random_r ()
  {
    return r * pow (frandom11 (), 1.0 / 3);	// rozklad kwadratowy
  }


  ///////////////////////////////////////////////////////////////////////////////
  // local Fermi momentum for particle (pdg and position dependent)
  ///////////////////////////////////////////////////////////////////////////////
  double localkf (particle & pa)
  {
    return localkf (pa.pdg, pa.r.length ());
  }

  ///////////////////////////////////////////////////////////////////////////////
  // local Fermi momentum from pdg code and  dist r from nucleus center 
  ///////////////////////////////////////////////////////////////////////////////
  double localkf (int pdg, double r)
  {
    switch (p * 100 + n)
      {
      case 100:
	return 0;		//wod�r
      default:
	return kf_from_density (density (pdg, r));
      }
    return kf;
  }


  ///////////////////////////////////////////////////////////////////////////////
  // local Fermi momentum from density 
  ///////////////////////////////////////////////////////////////////////////////
  double kf_from_density (double dens)
  {				// if(dens<=0) return 0;
    return pow (3 * Pi * Pi * dens * (fermi * fermi * fermi) / 2,
		1. / 3.) * 197.4 * MeV;
  }



  ///////////////////////////////////////////////////////////////////////////////
  // get nucleon with (random) momentum not exceeding localkf(r)
  ///////////////////////////////////////////////////////////////////////////////
  particle shoot (vec r)
  {				//zmieni�
    return get_nucleon (r);	// dummy r dependence
  }
  ///////////////////////////////////////////////////////////////////////////////
  //  get nucleon with random position and momentum not exceeding localkf(r)
  ///////////////////////////////////////////////////////////////////////////////
  particle shoot ()
  {
    double r = get_random_r ();
    return get_nucleon (r * rand_dir ());
  }

  ///////////////////////////////////////////////////////////////////////////////
  //  get nucleon with (random) momentum not exceeding localkf(r)
  ///////////////////////////////////////////////////////////////////////////////
  particle get_nucleon (vec r)
  {
    particle p0;
    p0.r=r;
    if (p + n == 1)
      {
	if (p == 1)
	  p0.set_proton ();
	else
	  p0.set_neutron ();
	return p0;
      }

    double x = frandom ();
    if (x * (p + n) < p)
      p0.set_proton ();
    else
      p0.set_neutron ();
    double lkf;

    if (local_kf == 0)
      lkf = kf;
    else
      lkf = localkf (p0.pdg, r.length ());
//    cout<<"lkf=="<<lkf/MeV<<endl; 
//    if(bodek)
//      p0.set_momentum(bodek_rand_from_ball(lkf));
//    else
    p0.set_momentum (rand_from_ball (lkf));
    return p0;
  }

  ///////////////////////////////////////////////////////////////////////////////
  // remove nucleon P from the nucleus 
  ///////////////////////////////////////////////////////////////////////////////
  void remove_nucleon (particle P)
  { // Volume and radius do not change; density decreases
    switch (P.pdg)
      {
      case pdg_proton:
	p--;
	break;
      case pdg_neutron:
	n--;
	break;
      default:
	return;
      }
    missingp4 = missingp4 + P.p4 ();
//    cout << "p=" << p << "n=" << n << endl;
  }

  ///////////////////////////////////////////////////////////////////////////////
  // insert nucleon P into the nucleus 
  ///////////////////////////////////////////////////////////////////////////////
  void insert_nucleon (particle P)
  { // Volume and radius do not change; density increases
    switch (P.pdg)
      {
      case pdg_proton:
	p++;
	break;
      case pdg_neutron:
	n++;
	break;
      default:
	return;
      }
    missingp4 = missingp4 - P.p4 ();
  }
  ///////////////////////////////////////////////////////////////////////////////
  // check if particle is Pauli blocked - depends on localkf at particle position
  ///////////////////////////////////////////////////////////////////////////////
  bool pauli_blocking (particle & pa)
  {
    return pauli_blocking (pa.pdg, pa.momentum (), pa.r.length ());
  }

  ///////////////////////////////////////////////////////////////////////////////
  // check if any of p[0]...p[n-1] is Pauli blocked 
  ///////////////////////////////////////////////////////////////////////////////
  bool pauli_blocking (particle p[], int n)
  {
    for (int i = 0; i < n; i++)
      if (pauli_blocking (p[i]))
	{
	  // cout << "PAULI BLOCKING" << endl;
	  return true;
	}
    return false;
  }

  ///////////////////////////////////////////////////////////////////////////////
  // check if particle def by pdg and momentum is blocked at dist r from center
  ///////////////////////////////////////////////////////////////////////////////
  bool pauli_blocking (int pdg, double p, double r)	// TRUE = PARTICLE IS BLOCKED
  {
    if (p + n == 1)
      return false;
    if (local_kf == 0)
      return p < kf;
    else
      return p < localkf (pdg, r);
  }
  ///////////////////////////////////////////////////////////////////////////////
  // nucleon effective mass in the cuclear matter ar dist r from center
  ///////////////////////////////////////////////////////////////////////////////
  double mstar (double r)      // not used probably
  {				
    return mass_proton;       // DUMMY - assume unchanged
  }



};
#endif
