#ifndef _particle_h_
#define _particle_h_
#include <iostream>
#include <cassert>
#include <cmath>
#include "generatormt.h"
#include "vect.h"
#include "pdg.h"


using namespace PDG; 

/// elementary particle on a mass shell is represented as a lorentz fourvector
/// and a pdg number. the position r-is included for tracting it in the nucleous
/// ks and orgi are introduced by Jarek for compatibility with the HEP-event structure
class particle : public vect
{

  double _mass;    ///< on shell mass
public:
  vect r;	   ///< position and time relative to the centre of the Nucleus event start time
  int pdg;         ///< pdg code of the particle  
  char ks;         ///< from HEP event
  char orgin;      ///< from HEP event
  double travelled;///< distance from creation
  int mother;
  int endproc;
public:
inline particle(){x=y=z=t=_mass=pdg=0;}
inline particle (int code,double mass);      ///< create particle at rest 
inline particle (double mass);               ///< create particle at rest 
inline particle (vect fourmomentum);         ///< create moving particle
inline double E ();                          ///< particle energy 
inline double energy ();                     ///< particle energy 
inline double Ek();                          ///< particle kinetic energy 
inline double mass ();                       ///< particle mass    
inline double m ();                          ///< particle mass    
inline int charge();                         ///< particle charge
inline double mass2 ();                      ///< mass squared
inline void set_mass (double mass);          ///< set particle mass and adjust energy
inline void set_pdg_and_mass (int pdg, double mass); ///< set particle pdg and mass
inline void set_proton (){set_pdg_and_mass(pdg_proton,mass_proton);}    ///< set particle pdg and mass
inline void set_neutron (){set_pdg_and_mass(pdg_neutron,mass_neutron);} ///< set particle pdg and mass
inline void set_pi (){set_pdg_and_mass(pdg_pi,mass_pi);}                ///< set particle pdg and mass
inline void set_piP (){set_pdg_and_mass(pdg_piP,mass_piP);}             ///< set particle pdg and mass
inline void set_piM (){set_pdg_and_mass(-pdg_piP,mass_piP);}            ///< set particle pdg and mass

inline void set_momentum (vec p);            ///< set particle momentum and adjust energy
inline void set_energy (double E);           ///< set particle energy and adjust momentum
inline vec p();                              ///< the momentum as a 3-vector
inline vect& p4();                           ///< the four-momentum 
inline vec v();                              ///< the velocity as a 3-vector
inline void krok_czasowy(double dt);         ///< move the particle by v*dt                  
inline void krok(double dl);                 ///< move the particle by dl                   
inline double momentum ();                   ///< value of the momentum 
inline double momentum2 ();                  ///< momenutm squared
inline bool is_valid();                             ///< checkes for nan in energy an momentum
inline bool decay (particle & p1, particle & p2);   ///< cause particle to decay into p1 and p2
                                             ///< (preserves the masses of p1 and p2)   

friend ostream & operator<<(ostream & out,particle p)
{
   return out<<"["<<p.pdg<<"/"<<p._mass<<"]"<<vect(p)<<"@"<<p.r/fermi;
}
};

bool decay(vect v, particle & p1, particle & p2);  /// the total momentum is enough

bool particle::decay (particle & p1, particle & p2)
{
 return ::decay(p4(),p1,p2);
}



double relative_speed(particle & a,particle &b);	///<

double relative_energy(particle & a,particle &b);	///<

double get_cos(double A, double B);    ///< zmienna o rozk�adzie prop. do (A*x+B)*x*x*x+1
/////////////////////////////////////////////////////////////////////////////////////
//                I M P L E M E N T A C J A
/////////////////////////////////////////////////////////////////////////////////////

particle::particle (int code, double mass):vect(mass,0,0,0),_mass (mass),pdg(code)
  {
    mother=0;endproc=-1;  
  }


particle::particle (double mass):vect(mass,0,0,0),_mass (mass)
  {
    mother=0;endproc=-1;  
  }
particle::particle (vect fourmomentum):vect (fourmomentum)
  { 
    _mass = sqrt (fourmomentum * fourmomentum);
    mother=0;endproc=-1;  
  }

//    particle(int code):_code(code){}

double particle::energy ()
  {
    return t;
  }

double particle::E()
  {
    return t;
  }

double particle::Ek()
  {
    return t-_mass;
  }

double particle::mass ()
  {
    return _mass;
  }

double particle::m ()
  {
    return _mass;
  }

int particle::charge ()
  {
    return PDG::charge(pdg);
  }

double particle::mass2 ()
  {
    return _mass*_mass;
  }

void particle::set_mass (double mass)
  {
    _mass = mass;
    t=sqrt(x*x+y*y+z*z+_mass*_mass);
  }

void particle::set_pdg_and_mass (int pdg, double mass)
  { this->pdg=pdg;
    _mass = mass;
    t=sqrt(x*x+y*y+z*z+_mass*_mass);
  }


void particle::set_momentum (vec p)
  {
    x=p.x;
    y=p.y;
    z=p.z;
    t = sqrt (p * p + _mass * _mass);
  }
 
  // adjust momentum so that to obtain the given energy
  // on the mass shell
void particle::set_energy (double E)
  {
    t=E; 
    double k=sqrt((E*E-_mass*_mass)
                 /(x*x+y*y+z*z)
		 );
    x*=k;
    y*=k;
    z*=k;
  }

vec particle::p ()
  {
    return vec(x,y,z);
  }

vect& particle::p4 ()
  {
    return *this;
  }

vec particle::v ()
  {
    return vec(x/t,y/t,z/t);
  }

double particle::momentum ()
  {
    return vec(x,y,z).length();
  }

double particle::momentum2 ()
  {
    return x*x+y*y+z*z;
  }

bool particle::is_valid () ///< no nan errors in mass and 4-momentum
{ return _mass==_mass && x==x && y==y && z==z && t==t;
}

void particle::krok(double dl)
{ 
  double V=v().length(); 
  r+=vect(dl*v()/V, dl/V);
  travelled+=dl;
}

void particle::krok_czasowy(double dt)
{  
  r+=vect(dt*v(),dt);
  travelled+=v().length()*dt;
}

#endif
