#include "scatter.h"
#include "particle.h"
#include <TGenPhaseSpace.h>

/// transate root TLorentzVector to vect  
static inline vect makevect (TLorentzVector * v)
{
  return vect (v->T (), v->X (), v->Y (), v->Z ());
}

/// Scatter p1,p2 into p3,p4  isotropically in cms 
// if f is given returns f(s,q2) for chosen kinematics
double
scatter (particle p1, particle p2,
	 particle & p3, particle & p4, 
	 double (*f) (double, double))
{
  vect suma = vect (p1) + vect (p2);
  if (suma * suma < 0)
    {
      cerr << "PROBLEM: cms speed >1  (=" << suma.v ().length ();
      cerr << ")" << p1.v ().length () << ' ' << p2.v ().length () << endl;
      cerr << "p1=" << vect (p1) << " p2=" << vect (p2) << " p1+p3=" << suma << endl;
      return 0;
    }
  if (!decay (suma, p3, p4))
{cerr << "The process is kinematically impossible because"<<endl;
cerr << "suma ="<<vect (suma)<<"  p3 ="<< vect (p3)<<"  p4 ="<<vect (p4)<<endl;
    return 0;}
  vect p13 = p1 - p3;
  double q2 = p13 * p13;
  if (f)
    return (*f) (suma * suma, q2);
  else
    return q2;
}

/// Scatter p1,p2 into p3,p4 according to distribution 
/// given by f=Ax^3+Bx where x=\cos \theta
int
scatterAB (particle p1, particle p2, 
	   particle & p3, particle & p4, 
	   double A, double B)
{
//  echo (scatterAB);
  vect total4momentum = vect (p1) + vect (p2);
  if (total4momentum * total4momentum < 0)
    {
      cerr << "PROBLEM: cms speed >1  (=" << total4momentum.v ().length ();
      cerr << ")" << p1.v ().length () << ' ' << p2.v ().length () << endl;
      cerr << "p1=" << vect (p1) << " p2=" << vect (p2) << " p1+p3=" <<	total4momentum << endl;
      return 0;
    }

  double MM = total4momentum * total4momentum;
  if (sqrt (MM) < p3.mass () + p4.mass ())
  {
//     cerr<<"za ma�a masa na ten rozpad"<<endl;
     return 0;// 0 means failure
  }
  vec v = total4momentum.v ();
  vect P1 = p1;
//  cout<<"P1="<<P1<<endl;
  P1.boost (-v);		// now the direction of P1 is the 'z'-axis
//  cout<<"P1="<<P1<<endl;
  double oldp = vec (P1).length ();
  if (oldp == 0)
  {  
    cerr<<"scatterAB: za ma�a masa"<<endl;
    return 0; // 0 means failure
  }
//    cout<<"sqrt(MM)="<<sqrt(MM)<<endl;
  double m1m1 = p3.mass () * p3.mass ();
  double m2m2 = p4.mass () * p4.mass ();
  double pp =
    (MM * MM + m1m1 * m1m1 + m2m2 * m2m2 -
     2 * (MM * m1m1 + MM * m2m2 + m1m1 * m2m2)) / (4 * MM);
//    cout<<"pp="<<pp<<endl;   
  double p = sqrt (pp);
  double kos = get_cos (A, B);
  vec a = vec (P1).dir ();
  vec b, c;
  if (fabs (a.x) > fabs (a.y))
    b = vec (a.z, 0, -a.x).dir ();	// b  prostopad�y do P1 
  else
    b = vec (0, a.z, -a.y).dir ();	// b  prostopad�y do P1 
  c = vecprod(a,b);
  double x = frandom () * 2 * M_PI;
  double pr = p * kos;
  double ppr = p * sqrt (1 - kos * kos);
  vec P3 = pr * a + ppr * sin (x) * b + ppr * cos (x) * c;
  p3.set_momentum (P3);
  p4.set_momentum (-P3);
  p3.boost (v);
  p4.boost (v);
  return 1;  // 1 means success
}


/// Scatter p1,p2 into p[0],..,p[n-1] uniformly in phase space
int scatter (int n, particle p1, particle p2, 
		    particle p[])
  { 
////  echo (scatter);
////  cout <<"p1="<<p1<<endl;
////  cout <<"p2="<<p2<<endl;

    vect in = p1 + p2;

    double masses[4];
    double sm = 0;
    for (int i = 0; i < n; i++)
      {
	sm += masses[i] = p[i].mass ();
      }

    if (sm * sm > in * in)
      {
//	cerr<< "Za ma�a masa ca�kowita  na ten rozpad"<<endl;
//        cerr<<"in*in="<<in*in<<endl;
//        cerr<<"out*out="<<sm*sm<<endl;
	return 0;
      }

    TGenPhaseSpace event;
    TLorentzVector v1 (in.x, in.y, in.z, in.t);
    event.SetDecay (v1, n, masses); // give the total momentum, number of particles, 
                                    // and masses of particles in the final state

    double x;
    do	// prawdopodobienstwo kazdego rozkladu powinno byc jednakowe
      {
	x = event.Generate ();
      }
    while (x < frandom11 () * event.GetWtMax ());

    for (int i = 0; i < n; i++)
      {
	p[i].p4 () = makevect (event.GetDecay (i));
	p[i].r = p1.r;
     if(!p[i].is_valid())
	{
	  cerr << p[i];
////	  echo (ERROR ERROR ERROR);
	}
      }
////    echo (scatter done);
    return 1;
  }


