 #include "CGaussianSF_O16.h"

//Gaussian approximation of the spectral function

CGaussianSF_O16::CGaussianSF_O16(const MomDistribs i_momDistrib, 
						 const IsospinOfSF i_isospin, 
						 CRandomGen& io_rnd)
			:m_rnd(io_rnd),
			m_isospin(i_isospin),
			m_targetBeta( (oxygenA-2.0)/(oxygenA-1.0) ),
			m_targetE2(oxygenE2),
			m_pBlock(oxygenPBlock),
			m_pStart(0.0),
			m_pStop(5.0*hbarc),
			m_eStart(0.0),
			m_eStop(5.0*hbarc)
{
	m_momDistrib = createDistrib(i_momDistrib, m_isospin, io_rnd);

	switch (i_momDistrib)
	{
		case md_O16_CdA:
			m_targetAlpha = 3.0/(4.0*m_targetBeta*avPMF_O16_CdA*avPMF_O16_CdA);
			break;

		case md_O16_Ben:
			m_targetAlpha = 3.0/(4.0*m_targetBeta*avPMF_O16_Ben*avPMF_O16_Ben);
			break;

		case md_O16_GCo:
			m_targetAlpha = 3.0/(4.0*m_targetBeta*avPMF_O16_GCo*avPMF_O16_GCo);
			break;

		case md_C12_Ben:
			m_targetAlpha = 3.0/(4.0*m_targetBeta*avPMF_C12_Ben*avPMF_C12_Ben);
			break;
			
		case md_Ca40_CdA:
			m_targetAlpha = 3.0/(4.0*m_targetBeta*avPMF_Ca40_CdA*avPMF_Ca40_CdA);
			break;

		case md_Ca40_GCo:
			if (m_isospin == proton) 
				m_targetAlpha = 3.0/(4.0*m_targetBeta*avPMF_Ca40P_GCo*avPMF_Ca40P_GCo);
			else
				m_targetAlpha = 3.0/(4.0*m_targetBeta*avPMF_Ca40N_GCo*avPMF_Ca40N_GCo);
			break;

		case md_Ca48_GCo:
			if (m_isospin == proton) 
				m_targetAlpha = 3.0/(4.0*m_targetBeta*avPMF_Ca48P_GCo*avPMF_Ca48P_GCo);
			else
				m_targetAlpha = 3.0/(4.0*m_targetBeta*avPMF_Ca48N_GCo*avPMF_Ca48N_GCo);
			break;

		case md_Fe56_Ben:
			m_targetAlpha = 3.0/(4.0*m_targetBeta*avPMF_Fe56_Ben*avPMF_Fe56_Ben);
			break;
	}

	m_coeff = M*sqrt(m_targetAlpha/pi);

	int iCnt(0);

	switch (m_isospin)
	{
		case proton:
			for(iCnt=0;iCnt<numberOfOxLevels;++iCnt)
			{
				m_TEnergy[iCnt] = TOxygenPEnergy[iCnt];
				m_TWidth[iCnt]  = TOxygenPWidth[iCnt];
			}
			break;

		case neutron:
			for(iCnt=0;iCnt<numberOfOxLevels;++iCnt)
			{
				m_TEnergy[iCnt] = TOxygenNEnergy[iCnt];
				m_TWidth[iCnt]  = TOxygenNWidth[iCnt];
			}
			break;
	};


	 m_1sShell1_2 = new CGaussianDistrib(m_TEnergy[oxLev1s1_2],m_TWidth[oxLev1s1_2],io_rnd);
	 m_1pShell3_2 = new CGaussianDistrib(m_TEnergy[oxLev1p3_2],m_TWidth[oxLev1p3_2],io_rnd);
	 m_1pShell1_2 = new CGaussianDistrib(m_TEnergy[oxLev1p1_2],m_TWidth[oxLev1p1_2],io_rnd);

}



double CGaussianSF_O16::evalSF(const double p, const double removalE) const
{

	const double mf = m_momDistrib->MF(p)*(2.0*m_1sShell1_2->eval(removalE) + 4.0*m_1pShell3_2->eval(removalE) + 2.0*m_1pShell1_2->eval(removalE) )/oxygenZ;

	if ( p == 0.0 )
		return mf;

	double squareRoot( 2.0*M*m_targetBeta*(removalE - m_targetE2) );

	if(squareRoot<0.0)
		return mf;

	squareRoot = sqrt(squareRoot);

	const double pMinimum(m_targetBeta*p - squareRoot);
	const double pMaximum(m_targetBeta*p + squareRoot);

	const double corr = m_coeff/p*m_momDistrib->Corr(p)*( exp(-m_targetAlpha*pMinimum*pMinimum) - exp(-m_targetAlpha*pMaximum*pMaximum) );

	return mf + corr;

}


double CGaussianSF_O16::evalMFMomD(const double p) const
{
	return m_momDistrib->MF(p);
}


double CGaussianSF_O16::evalCorrMomD(const double p) const
{
	return m_momDistrib->Corr(p);
}


double CGaussianSF_O16::evalTotMomD(const double p) const
{
	return m_momDistrib->Tot(p);
}


double CGaussianSF_O16::generateMom() const
{
	return m_momDistrib->generate();
}



double CGaussianSF_O16::get_pBlock() const
{
	return m_pBlock;
}


double CGaussianSF_O16::get_pMin() const
{
	return m_pStart;
}


double CGaussianSF_O16::get_pMax() const
{
	return m_pStop;
}


double CGaussianSF_O16::get_eMin() const
{
	return m_eStart;
}


double CGaussianSF_O16::get_eMax() const
{
	return m_eStop;
}

double CGaussianSF_O16::get_targetAlpha() const
{
	return m_targetAlpha;
}
