#include "CSFOptions.h"


CSFOptions::CSFOptions(const TargetElement i_target,
		const IsospinOfSF i_isospin,
		const MomDistribs i_momDistrib,
		const ScatteredNeutrinoFlav i_neutrinoFlavour,
		const FormFactors i_formFactors,
		const bool i_switchToEM,
		const bool i_switchPauliBlocking,
		const double i_neutrinoE,
		const int i_numberOfRepetitions)
		 :m_target(i_target),
		 m_isospin(i_isospin),
		 m_momDistrib(i_momDistrib),
		 m_formFactors(i_formFactors),
		 m_switchToEM(i_switchToEM),
		 m_switchPauliBlocking(i_switchPauliBlocking),
		 m_neutrinoE(i_neutrinoE),
		 m_numberOfRepetitions(i_numberOfRepetitions)
{

	switch(i_target)
	{
		case targC_Ben:
			m_Z = carbonZ;
			m_N = carbonN;
			break;

		case targO_Ben:
			m_Z = oxygenZ;
			m_N = oxygenN;
			break;

		case targO_GSF:
			m_Z = oxygenZ;
			m_N = oxygenN;
			break;

		case targCa_GSF:
			m_Z = calciumZ;
			m_N = calciumN;
			break;

		case targAr_default:
			m_Z = argonZ;
			m_N = argonN;
			break;

		case targAr_GSF:
			m_Z = argonZ;
			m_N = argonN;
			break;

		case targFe_Ben:
			m_Z = ironZ;
			m_N = ironN;
			break;

		default:
			std::cout<<"There is no such target implemented"<<std::endl;
			return;
	};


	switch(i_neutrinoFlavour)
	{
		case flav_e:
			m_leptMass = eMass;
			m_leptMass2 = eMass*eMass;
			m_leptFlav = "electron";
			m_switchAntineut = false;
			break;

		case flav_ae:
			m_leptMass = eMass;
			m_leptMass2 = eMass*eMass;
			m_leptFlav = "electron";
			m_switchAntineut = true;
			break;

		case flav_mu:
			m_leptMass = muMass;
			m_leptMass2 = muMass*muMass;
			m_leptFlav = "muon";
			m_switchAntineut = false;
			break;
		
		case flav_amu:
			m_leptMass = muMass;
			m_leptMass2 = muMass*muMass;
			m_leptFlav = "muon";
			m_switchAntineut = true;
			break;

		case flav_tau:
			m_leptMass = tauMass;
			m_leptMass2 = tauMass*tauMass;
			m_leptFlav = "tau";
			m_switchAntineut = false;
			break;

		case flav_atau:
			m_leptMass = tauMass;
			m_leptMass2 = tauMass*tauMass;
			m_leptFlav = "tau";
			m_switchAntineut = true;
			break;
		
		case flav_mless:
			m_leptMass = 0.0;
			m_leptMass2 = 0.0;
			m_leptFlav = "fictitious";
			m_switchAntineut = false;
			break;

		case flav_amless:
			m_leptMass = 0.0;
			m_leptMass2 = 0.0;
			m_leptFlav = "fictitious";
			m_switchAntineut = true;
			break;

		default:
			std::cout<<"There is no such neutrino flavour"<<std::endl;
			return;
	};


	switch(i_formFactors)
	{
		case dipoleFF:
			FFGE = &DipoleGE;
			FFGM = &DipoleGM;
			break;
			
		case BBBA05FF:
			FFGE = &BBBA05_GE;
			FFGM = &BBBA05_GM;
			break;
			
		case BBA03FF:
			FFGE = &BBA03_GE;
			FFGM = &BBA03_GM;
			break;
			
		case JLabFF:
			FFGE = &JLabGE;
			FFGM = &JLabGM;
			break;

		default:
			std::cout<<"There are no such parameterization of the form factors"<<std::endl;
			return;

	};

}



TargetElement CSFOptions::get_target() const
{
	return m_target;
}


IsospinOfSF CSFOptions::get_isospin() const
{
	return m_isospin;
}


MomDistribs CSFOptions::get_momDistrib() const
{
	return m_momDistrib;
}



double CSFOptions::get_leptonMass() const
{
	return m_leptMass;
}


double CSFOptions::get_leptonMass2() const
{
	return m_leptMass2;
}


std::string CSFOptions::get_neutrinoFlav() const
{
	
	return m_leptFlav;
}


bool CSFOptions::get_switchAntineut() const
{
	return m_switchAntineut;
}



DFormFactorPtr CSFOptions::get_FFGE() const
{
	return FFGE;
}


DFormFactorPtr CSFOptions::get_FFGM() const
{
	return FFGM;
}



int CSFOptions::get_numberOfRepetitions() const
{
	return m_numberOfRepetitions;
}



bool CSFOptions::get_switchToEM() const
{
	return m_switchToEM;
}


bool CSFOptions::get_switchPauliBlocking() const
{
	return m_switchPauliBlocking;
}



double CSFOptions::get_neutrinoE() const
{
	return m_neutrinoE;
}



double CSFOptions::evalLH(const double q4til2, 
						   const double p4k4, 
						   const double p4kPrime4, 
						   const double p4q4til,
						   const double k4q4til,
						   const double kPrime4q4til,
						   const double k4kPrime4) const
{

	const double qM(q4til2/M2);
	const double tau(-qM/4.0);
	
	if (!m_switchToEM)
	{
		const double ge(FFGE(q4til2));
		const double gm(FFGM(q4til2));

		const double f1( (ge + tau*gm)/(1 + tau) );
		const double f2( (gm - ge)/(1 + tau) );

		const double fa( FA(q4til2) );
		const double fp( 2.0*M2*fa/(piMass2 - q4til2) );

		const double f11( f1*f1 );
		const double f22( f2*f2 );
		const double fa2( fa*fa );
		const double fp2( fp*fp );

		const double ff( f1 + f2 );

		const double h1( fa2*(1.0+tau) + ff*ff*tau );
		const double h2( fa2 + f11 + f22*tau );
		const double h3( 2.0*fa*ff );
		const double h4( 0.25*f22*(1.0-tau) + 0.5*f1*f2 + fa*fp - fp2*tau );

		const double lh( 2.0*( 2.0*M2*k4kPrime4*h1 + (2.0*p4k4*p4kPrime4-M2*k4kPrime4+p4k4*kPrime4q4til+p4kPrime4*k4q4til-k4kPrime4*p4q4til)*h2 + (p4kPrime4*k4q4til-p4k4*kPrime4q4til)*h3 + (k4kPrime4*q4til2-2.0*k4q4til*kPrime4q4til)*h4 ) );

		const int nuclNumb( m_N );

		return  nuclNumb*lh;
	

/*		const double a = M2*(m_leptMass2 - q4til2)/4.0*(fa2*(4.0 - qM) - f11*(4.0 + qM) - qM*f22*(1.0 - tau) - 4.0*qM*f1*f2 - m_leptMass2/M2*(ff*ff + fa2 + 4*fa*fp + qM*fp2));
		const double b = -q4til2*ff*fa;//M2*B(q2)
		const double c = 0.25*(f11 + tau*f22 + fa2);
	
		const double su = 4.0*p4k4 + q4til2 - m_leptMass2;
		
		const int nuclNumb( m_N );
		
		return  nuclNumb*(a + su*( c*su -(!m_switchAntineut)*b +(m_switchAntineut)*b ));*/
	

	}
	


//-----------------------------------------------proton---------------------------------------------
	switch(m_formFactors)
	{
		case dipoleFF:
			FFGE = &ProtonDipoleGE;
			FFGM = &ProtonDipoleGM;
			break;
			
		case BBBA05FF:
			FFGE = &ProtonBBBA05_GE;
			FFGM = &ProtonBBBA05_GM;
			break;
			
		case BBA03FF:
			FFGE = &ProtonBBA03_GE;
			FFGM = &ProtonBBA03_GM;
			break;
			
		case JLabFF:
			FFGE = &ProtonJLabGE;
			FFGM = &ProtonJLabGM;
			break;

	};


	double ge( FFGE(q4til2) );
	double gm( FFGM(q4til2) );

	double f1( (ge + tau*gm)/(1 + tau) );
	double f2( (gm - ge)/(1 + tau) );

	double f11( f1*f1 );
	double f22( f2*f2 );

	double ff( f1 + f2 );

	double h1( ff*ff*tau );
	double h2( f11 + f22*tau );
	double h4( 0.25*f22*(1.0-tau) + 0.5*f1*f2 );

	double lh( 2.0*( 2.0*M2*k4kPrime4*h1 + (2.0*p4k4*p4kPrime4-M2*k4kPrime4+p4k4*kPrime4q4til+p4kPrime4*k4q4til-k4kPrime4*p4q4til)*h2 + (k4kPrime4*q4til2-2.0*k4q4til*kPrime4q4til)*h4 ) );

	double protResult = lh;
//-----------------------------------------end of proton---------------------------------------------

//-----------------------------------------------neutron---------------------------------------------
	switch(m_formFactors)
	{
		case dipoleFF:
			FFGE = &NeutronDipoleGE;
			FFGM = &NeutronDipoleGM;
			break;
			
		case BBBA05FF:
			FFGE = &NeutronBBBA05_GE;
			FFGM = &NeutronBBBA05_GM;
			break;
			
		case BBA03FF:
			FFGE = &NeutronBBA03_GE;
			FFGM = &NeutronBBA03_GM;
			break;
			
		case JLabFF:
			FFGE = &NeutronJLabGE;
			FFGM = &NeutronJLabGM;
			break;

	};

	ge = FFGE(q4til2);
	gm = FFGM(q4til2);

	f1 = (ge + tau*gm)/(1 + tau);
	f2 = (gm - ge)/(1 + tau);

	f11 = f1*f1;
	f22 = f2*f2;

	ff = f1 + f2;

	h1 = ff*ff*tau;
	h2 = f11 + f22*tau;
	h4 = 0.25*f22*(1.0-tau) + 0.5*f1*f2;

	lh = 2.0*( 2.0*M2*k4kPrime4*h1 + (2.0*p4k4*p4kPrime4-M2*k4kPrime4+p4k4*kPrime4q4til+p4kPrime4*k4q4til-k4kPrime4*p4q4til)*h2 + (k4kPrime4*q4til2-2.0*k4q4til*kPrime4q4til)*h4 );
//-----------------------------------------end of neutron---------------------------------------------

	return  (lh*m_N + protResult*m_Z)/(q4til2*q4til2);
	

	
}
