#ifndef CVEC_H
#define CVEC_H

#include <iostream>
#include <iomanip>
#include <cmath>

#include "CRandomGen.h"

const double valOfPi(4.0*atan(1.0));

class CVec
{
public:
	CVec(const double i_x, const double i_y, const double i_z);
	CVec(const double i_maximumNorm, CRandomGen& io_rnd, const bool onSphere);
	CVec(const double i_deltaNorm, const double i_minimalNorm, CRandomGen& io_rnd);

	inline void set_x(const double i_x);
	inline void set_y(const double i_y);
	inline void set_z(const double i_z);

	inline double get_x() const;
	inline double get_y() const;
	inline double get_z() const;

	inline double normSq() const;
	inline double norm() const;

	inline CVec operator+ (const CVec& rhs) const; 
	inline CVec operator- (const CVec& rhs) const;
	inline CVec operator- () const;
	inline double operator* (const CVec& rhs) const;
	inline CVec operator* (const double& rhs) const;
	inline CVec operator/ (const double& mult) const;
	
	friend std::ostream& operator<< (std::ostream& o_stream, const CVec& rhs);
	friend CVec operator* (const double& lhs, const CVec& rhs);


private:
	union
	{	
		struct
		{
			mutable double m_x;
			mutable double m_y;
			mutable double m_z;
		};
		double m_v[3];
	};

	mutable double m_r2;
	mutable double m_r;
};



//constructors and destructor of the CVec class:
CVec::CVec(const double i_x = 0.0, const double i_y = 0.0, const double i_z = 0.0)
:m_x(i_x),
 m_y(i_y),
 m_z(i_z),
 m_r2(m_x*m_x + m_y*m_y + m_z*m_z),
 m_r(sqrt(m_r2))
{
}

CVec::CVec(const double i_maximalNorm, CRandomGen& io_rnd, const bool onSphere)
{
	m_r = (onSphere)*i_maximalNorm + !(onSphere)*i_maximalNorm*io_rnd;
	m_r2 = m_r*m_r;

	const double phi( 2.0*valOfPi*io_rnd );
	const double cosTheta( 2.0*io_rnd - 1.0 );

	const double rsinTheta( m_r*sqrt(1.0 - cosTheta*cosTheta) );

	m_x = rsinTheta*cos(phi);
	m_y = rsinTheta*sin(phi);
	m_z = m_r*cosTheta;
}

CVec::CVec(const double i_deltaNorm, const double i_minimalNorm, CRandomGen& io_rnd)
{
	m_r = i_deltaNorm*io_rnd + i_minimalNorm;
	m_r2 = m_r*m_r;

	const double phi( 2*valOfPi*io_rnd );
	const double cosTheta( 2*io_rnd - 1.0 );

	const double rsinTheta( m_r*sqrt(1.0 - cosTheta*cosTheta) );

	m_x = rsinTheta*cos(phi);
	m_y = rsinTheta*sin(phi);
	m_z = m_r*cosTheta;
}





//definitions of functions of the class:
inline void CVec::set_x(const double i_x)
{
	m_r2 -= m_x*m_x;

	m_x = i_x;

	m_r2 += i_x*i_x;
	m_r = sqrt(m_r2);
}
	
inline void CVec::set_y(const double i_y)
{
	m_r2 -= m_y*m_y;

	m_y = i_y;

	m_r2 += i_y*i_y;
	m_r = sqrt(m_r2);
}
	
inline void CVec::set_z(const double i_z)
{
	m_r2 -= m_z*m_z;

	m_z = i_z;

	m_r2 += i_z*i_z;
	m_r = sqrt(m_r2);


}



inline double CVec::get_x() const
{
	return m_x;
}
	
inline double CVec::get_y() const
{
	return m_y;
}
	
inline double CVec::get_z() const
{
	return m_z;
}


inline double CVec::normSq() const
{
	return m_r2; 
}

inline double CVec::norm() const
{
	return m_r; 
}

//operators
inline CVec CVec::operator+ (const CVec& rhs) const
{
	return CVec ( m_x + rhs.m_x, m_y + rhs.m_y, m_z + rhs.m_z );
}

inline CVec CVec::operator- (const CVec& rhs) const
{
	return CVec ( m_x - rhs.m_x, m_y - rhs.m_y, m_z - rhs.m_z );
}

inline CVec CVec::operator- () const
{
	return CVec ( -m_x, -m_y, -m_z );
}

inline double CVec::operator* (const CVec& rhs) const
{
	return ( m_x*rhs.m_x + m_y*rhs.m_y + m_z*rhs.m_z );
}

inline CVec CVec::operator* (const double& rhs) const
{
	return CVec ( m_x*rhs, m_y*rhs, m_z*rhs );
}

inline CVec CVec::operator/ (const double& rhs) const
{
	return CVec ( m_x/rhs, m_y/rhs, m_z/rhs );
}

std::ostream& operator<< (std::ostream& o_stream, const CVec& rhs)
{
	const int wid( o_stream.width() );
	const int prec( o_stream.precision() );

	o_stream<<std::fixed<<rhs.m_x<<" "<<std::showpoint<<std::fixed<<std::setw(wid)<<std::setprecision(prec)<<rhs.m_y<<" "<<std::showpoint<<std::fixed<<std::showpoint<<std::setw(wid)<<std::setprecision(prec)<<rhs.m_z; 
	
	return o_stream;
}

CVec operator* (const double& lhs, const CVec& rhs)
{
	return rhs*lhs;
}

#endif
